@@ -1,7 +1,7 @@
 /**
  * @file subscription.c  common subscription handling
  *
- * Copyright (C) 2003-2021 Lars Windolf <lars.windolf@gmx.de>
+ * Copyright (C) 2003-2025 Lars Windolf <lars.windolf@gmx.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,9 +29,11 @@
 #include "db.h"
 #include "debug.h"
 #include "feedlist.h"
+#include "itemlist.h"
 #include "metadata.h"
 #include "net.h"
 #include "subscription_icon.h"
+#include "xml.h"
 #include "ui/auth_dialog.h"
 #include "ui/feed_list_view.h"
 #include "ui/itemview.h"
@@ -41,7 +43,7 @@
 #define FEED_PROTOCOL_PREFIX "feed://"
 #define FEED_PROTOCOL_PREFIX2 "feed:"
 
-#define ONE_MONTH_MICROSECONDS (guint64)(60*60*24*31) * (guint64)G_USEC_PER_SEC
+#define ONE_MONTH_MICROSECONDS (gint64)(60*60*24*31) * (gint64)G_USEC_PER_SEC
 
 subscriptionPtr
 subscription_new (const gchar *source,
@@ -53,6 +55,8 @@ subscription_new (const gchar *source,
 	subscription = g_new0 (struct subscription, 1);
 	subscription->type = feed_get_subscription_type ();
 	subscription->updateOptions = options;
+	subscription->cacheLimit = CACHE_DEFAULT;
+	subscription->valid = TRUE;
 
 	if (!subscription->updateOptions)
 		subscription->updateOptions = g_new0 (struct updateOptions, 1);
@@ -114,10 +118,10 @@ subscription_can_be_updated (subscriptionPtr subscription)
 		return FALSE;
 	}
 
-	if (!subscription_get_source (subscription)) {
-		g_warning ("Feed source is NULL! This should never happen - cannot update!");
+	// can be the case for newsbins
+	if (!subscription_get_source (subscription))
 		return FALSE;
-	}
+	
 	return TRUE;
 }
 
@@ -128,7 +132,7 @@ subscription_reset_update_counter (subscriptionPtr subscription, guint64 *now)
 		return;
 
 	subscription->updateState->lastPoll = *now;
-	debug2 (DEBUG_UPDATE, "Resetting last poll counter of %s to %lld.", subscription->source, subscription->updateState->lastPoll);
+	debug (DEBUG_UPDATE, "Resetting last poll counter of %s to %lld.", subscription->source, subscription->updateState->lastPoll);
 }
 
 /**
@@ -169,10 +173,10 @@ subscription_update_error_status (subscriptionPtr subscription,
 }
 
 static void
-subscription_process_update_result (const struct updateResult * const result, gpointer user_data, guint32 flags)
+subscription_process_update_result (const UpdateResult * const result, gpointer user_data, guint32 flags)
 {
 	subscriptionPtr subscription = (subscriptionPtr)user_data;
-	nodePtr		node = subscription->node;
+	Node		*node = subscription->node;
 	gboolean	processing = FALSE;
 	guint		count, maxcount;
 	gchar		*statusbar;
@@ -183,13 +187,16 @@ subscription_process_update_result (const struct updateResult * const result, gp
 	g_assert (subscription->updateJob);
 	/* update the subscription URL on permanent redirects */
 	if ((301 == result->httpstatus || 308 == result->httpstatus) && result->source && !g_str_equal (result->source, subscription->updateJob->request->source)) {
-		debug2 (DEBUG_UPDATE, "The URL of \"%s\" has changed permanently and was updated to \"%s\"", node_get_title(node), result->source);
+		debug (DEBUG_UPDATE, "The URL of \"%s\" has changed permanently and was updated to \"%s\"", node_get_title(node), result->source);
 		subscription_set_source (subscription, result->source);
     statusbar = g_strdup_printf (_("The URL of \"%s\" has changed permanently and was updated"), node_get_title(node));
   }
 
 	/* consider everything that prevents processing the data we got */
-	if (result->httpstatus >= 400 || !result->data) {
+	if (304 == result->httpstatus) {
+		node->available = TRUE;
+		statusbar = g_strdup_printf (_("\"%s\" has not changed since last update"), node_get_title(node));
+	} else if (result->httpstatus >= 400 || !result->data) {
 		/* Default */
 		subscription->error = FETCH_ERROR_NET;
 		node->available = FALSE;
@@ -200,12 +207,9 @@ subscription_process_update_result (const struct updateResult * const result, gp
 			auth_dialog_new (subscription, flags);
 		}
 		if (410 == result->httpstatus) { /* gone */
-			subscription->discontinued = TRUE;
+			subscription_set_discontinued (subscription, TRUE);
 			statusbar = g_strdup_printf (_("\"%s\" is discontinued. Liferea won't updated it anymore!"), node_get_title (node));
 		}
-	} else if (304 == result->httpstatus) {
-		node->available = TRUE;
-		statusbar = g_strdup_printf (_("\"%s\" has not changed since last update"), node_get_title(node));
 	} else if (result->filterErrors) {
 		node->available = FALSE;
 		subscription->error = FETCH_ERROR_NET;
@@ -214,7 +218,7 @@ subscription_process_update_result (const struct updateResult * const result, gp
 	}
 
 	/* Clear status bar if we are last update in progress */
-	update_jobs_get_count (&count, &maxcount);
+	update_job_queue_get_count (&count, &maxcount);
 	if (1 >= count)
 		liferea_shell_set_status_bar (statusbar);
 	else
@@ -271,7 +275,7 @@ subscription_update (subscriptionPtr subscription, guint flags)
 	if (subscription->updateJob)
 		return;
 
-	debug1 (DEBUG_UPDATE, "Scheduling %s to be updated", node_get_title (subscription->node));
+	debug (DEBUG_UPDATE, "Scheduling %s to be updated", node_get_title (subscription->node));
 
 	if (subscription_can_be_updated (subscription)) {
 		now = g_get_real_time();
@@ -282,16 +286,17 @@ subscription_update (subscriptionPtr subscription, guint flags)
 			subscription->updateState,
 			subscription->updateOptions
 		);
+		update_request_allow_commands (request, TRUE);
 
 		if (subscription_get_filter (subscription))
 			request->filtercmd = g_strdup (subscription_get_filter (subscription));
 
 		if (SUBSCRIPTION_TYPE (subscription)->prepare_update_request (subscription, request))
-			subscription->updateJob = update_execute_request (subscription, request, subscription_process_update_result, subscription, flags);
+			subscription->updateJob = update_job_new (subscription, request, subscription_process_update_result, subscription, flags);
 		else
 			g_object_unref (request);
 
-		update_jobs_get_count (&count, &maxcount);
+		update_job_queue_get_count (&count, &maxcount);
 		if (count > 1)
 			liferea_shell_set_status_bar (_("Updating (%d / %d) ..."), maxcount - count, maxcount);
 		else
@@ -365,6 +370,12 @@ subscription_set_default_update_interval (subscriptionPtr subscription, guint in
 	subscription->defaultInterval = interval;
 }
 
+void
+subscription_set_discontinued (subscriptionPtr subscription, gboolean newState)
+{
+	subscription->discontinued = newState;
+}
+
 static const gchar *
 subscription_get_orig_source (subscriptionPtr subscription)
 {
@@ -473,6 +484,10 @@ subscription_import (xmlNodePtr xml, gboolean trusted)
 	if (!source)
 		source = xmlGetProp (xml, BAD_CAST "xmlurl");	/* e.g. for AmphetaDesk */
 
+	xmlChar	*typeStr = xmlGetProp (xml, BAD_CAST"type");
+	subscription->fhp = feed_type_str_to_fhp ((gchar *)typeStr);
+	xmlFree (typeStr);
+
 	if (source) {
 		if (!trusted && source[0] == '|') {
 			/* FIXME: Display warning dialog asking if the command
@@ -518,6 +533,46 @@ subscription_import (xmlNodePtr xml, gboolean trusted)
 		subscription->updateOptions->password = (gchar *)xmlGetProp (xml, BAD_CAST "password");
 	}
 
+	/* Set the feed cache limit */
+	tmp = xmlGetProp (xml, BAD_CAST "cacheLimit");
+	if (tmp && !xmlStrcmp (tmp, BAD_CAST"unlimited"))
+		subscription->cacheLimit = CACHE_UNLIMITED;
+	else
+		subscription->cacheLimit = common_parse_long ((gchar *)tmp, CACHE_DEFAULT);
+	xmlFree (tmp);
+
+	/* enclosure auto download flag */
+	tmp = xmlGetProp (xml, BAD_CAST"encAutoDownload");
+	if (tmp && !xmlStrcmp (tmp, BAD_CAST"true"))
+		subscription->encAutoDownload = TRUE;
+	xmlFree (tmp);
+
+	/* comment feed handling flag */
+	tmp = xmlGetProp (xml, BAD_CAST"ignoreComments");
+	if (tmp && !xmlStrcmp (tmp, BAD_CAST"true"))
+		subscription->ignoreComments = TRUE;
+	xmlFree (tmp);
+
+	tmp = xmlGetProp (xml, BAD_CAST"markAsRead");
+	if (tmp && !xmlStrcmp (tmp, BAD_CAST"true"))
+		subscription->markAsRead = TRUE;
+	xmlFree (tmp);
+
+	tmp = xmlGetProp (xml, BAD_CAST"html5Extract");
+	if (tmp && !xmlStrcmp (tmp, BAD_CAST"true"))
+		subscription->html5Extract = TRUE;
+	xmlFree (tmp);
+
+	tmp = xmlGetProp (xml, BAD_CAST"loadItemLink");
+	if (tmp && !xmlStrcmp ((xmlChar *)tmp, BAD_CAST"true"))
+		subscription->loadItemLink = TRUE;
+	xmlFree (tmp);
+
+	tmp = xmlGetProp (xml, BAD_CAST"alwaysShowInReducedMode");
+	if (tmp && !xmlStrcmp (tmp, BAD_CAST"true"))
+		subscription->alwaysShowInReduced = TRUE;
+	xmlFree (tmp);
+
 	return subscription;
 }
 
@@ -536,52 +591,128 @@ subscription_export (subscriptionPtr subscription, xmlNodePtr xml, gboolean trus
 	if (subscription_get_filter (subscription))
 		xmlNewProp (xml, BAD_CAST"filtercmd", BAD_CAST subscription_get_filter (subscription));
 
-	if(trusted) {
-		xmlNewProp (xml, BAD_CAST"updateInterval", BAD_CAST interval);
+	xmlNewProp (xml, BAD_CAST"updateInterval", BAD_CAST interval);
 
-		if (subscription->updateOptions->dontUseProxy)
-			xmlNewProp (xml, BAD_CAST"dontUseProxy", BAD_CAST"true");
+	if (subscription->updateOptions->dontUseProxy)
+		xmlNewProp (xml, BAD_CAST"dontUseProxy", BAD_CAST"true");
 
-		if (!liferea_auth_has_active_store ()) {
-			if (subscription->updateOptions->username)
-				xmlNewProp (xml, BAD_CAST"username", (xmlChar *)subscription->updateOptions->username);
-			if (subscription->updateOptions->password)
-				xmlNewProp (xml, BAD_CAST"password", (xmlChar *)subscription->updateOptions->password);
-		}
+	if (trusted) {
+		g_autofree gchar *cacheLimit = NULL;
+
+		if (subscription->cacheLimit >= 0)
+			cacheLimit = g_strdup_printf ("%d", subscription->cacheLimit);
+		if (subscription->cacheLimit == CACHE_UNLIMITED)
+			cacheLimit = g_strdup ("unlimited");
+		if (cacheLimit)
+			xmlNewProp (xml, BAD_CAST"cacheLimit", BAD_CAST cacheLimit);
+
+		if (subscription->encAutoDownload)
+			xmlNewProp (xml, BAD_CAST"encAutoDownload", BAD_CAST"true");
+
+		if (subscription->ignoreComments)
+			xmlNewProp (xml, BAD_CAST"ignoreComments", BAD_CAST"true");
+
+		if (subscription->markAsRead)
+			xmlNewProp (xml, BAD_CAST"markAsRead", BAD_CAST"true");
+
+		if (subscription->html5Extract)
+			xmlNewProp (xml, BAD_CAST"html5Extract", BAD_CAST"true");
+
+		if (subscription->loadItemLink)
+			xmlNewProp (xml, BAD_CAST"loadItemLink", BAD_CAST"true");
+
+		if (subscription->alwaysShowInReduced)
+			xmlNewProp (xml, BAD_CAST"alwaysShowInReducedMode", BAD_CAST"true");
+
+		debug (DEBUG_CACHE, "adding subscription: source=%s interval=%d cacheLimit=%s",
+			subscription_get_source (subscription),
+			subscription_get_update_interval (subscription),
+			(cacheLimit != NULL ? cacheLimit : ""));
 	}
 
 	g_free (interval);
 }
 
-void
-subscription_to_xml (subscriptionPtr subscription, xmlNodePtr xml)
-{
-	gchar	*tmp;
+// content scraping
 
-	xmlNewTextChild (xml, NULL, BAD_CAST "feedSource", (xmlChar *)subscription_get_source (subscription));
-	xmlNewTextChild (xml, NULL, BAD_CAST "feedOrigSource", (xmlChar *)subscription_get_orig_source (subscription));
+static void
+subscription_enrich_item_cb (const UpdateResult * const result, gpointer userdata, updateFlags flags) {
+	itemPtr item;
+	gchar	*article;
 
-	tmp = g_strdup_printf ("%d", subscription_get_default_update_interval (subscription));
-	xmlNewTextChild (xml, NULL, BAD_CAST "feedUpdateInterval", (xmlChar *)tmp);
-	g_free (tmp);
+	if (!result->data || result->httpstatus >= 400)
+		return;
 
-	tmp = g_strdup_printf ("%d", subscription->discontinued?1:0);
-	xmlNewTextChild (xml, NULL, BAD_CAST "feedDiscontinued", (xmlChar *)tmp);
-	g_free (tmp);
+	item = item_load (GPOINTER_TO_UINT (userdata));
+	if (!item)
+		return;
 
-	if (subscription->updateError)
-		xmlNewTextChild (xml, NULL, BAD_CAST "updateError", (xmlChar *)subscription->updateError);
-	if (subscription->httpError) {
-		xmlNewTextChild (xml, NULL, BAD_CAST "httpError", (xmlChar *)subscription->httpError);
+	article = xhtml_extract_from_string (result->data, result->source);
+	if (article) {
+		// Enable AMP images by replacing <amg-img> by <img>
+		gchar **tmp_split = g_strsplit(article, "<amp-img", 0);
+		gchar *tmp = g_strjoinv("<img", tmp_split);
+		g_strfreev (tmp_split);
+		g_free (article);
+		article = tmp;
 
-		tmp = g_strdup_printf ("%d", subscription->httpErrorCode);
-		xmlNewTextChild (xml, NULL, BAD_CAST "httpErrorCode", (xmlChar *)tmp);
-		g_free (tmp);
+		metadata_list_set (&(item->metadata), "richContent", article);
+		db_item_update (item);
+		itemlist_update_item (item);
+		g_free (article);
 	}
-	if (subscription->filterError)
-		xmlNewTextChild (xml, NULL, BAD_CAST "filterError", (xmlChar *)subscription->filterError);
+	item_unload (item);
+}
 
-	metadata_add_xml_nodes (subscription->metadata, xml);
+/**
+ * Checks content of an items source and tries to crawl content
+ */
+void
+subscription_enrich_item (subscriptionPtr subscription, itemPtr item)
+{
+	UpdateRequest *request;
+
+	if (!item->source) {
+		debug (DEBUG_PARSING, "Cannot HTML5-enrich item %s because it has no source!", item->title);
+		return;
+	}
+
+	// Don't enrich twice
+	if (NULL != metadata_list_get (item->metadata, "richContent")) {
+		debug (DEBUG_PARSING, "Skipping already HTML5 enriched item %s", item->title);
+		return;
+	}
+
+	// Fetch item->link document and try to parse it as XHTML
+	debug (DEBUG_PARSING, "Fetching HTML5 %ld %s : %s", item->id, item->title, item->source);
+	request = update_request_new (
+		item->source,
+		NULL,	// updateState
+		subscription->updateOptions	// Pass options of parent feed (e.g. password, proxy...)
+	);
+
+	update_job_new (subscription, request, subscription_enrich_item_cb, GUINT_TO_POINTER (item->id), UPDATE_REQUEST_NO_FEED);
+}
+
+
+guint
+subscription_get_max_item_count (subscriptionPtr subscription)
+{
+	gint	default_max_items;
+
+	switch (subscription->cacheLimit) {
+		case CACHE_DEFAULT: // -1
+			conf_get_int_value (DEFAULT_MAX_ITEMS, &default_max_items);
+			return default_max_items;
+			break;
+		case CACHE_DISABLE: // 0
+		case CACHE_UNLIMITED:
+			return G_MAXUINT;
+			break;
+		default: // any other positive value
+			return subscription->cacheLimit;
+			break;
+	}
 }
 
 void
@@ -597,6 +728,9 @@ subscription_free (subscriptionPtr subscription)
 	g_free (subscription->origSource);
 	g_free (subscription->filtercmd);
 
+	if (subscription->parseErrors)
+		g_string_free (subscription->parseErrors, TRUE);
+
 	update_job_cancel_by_owner (subscription);
 	update_options_free (subscription->updateOptions);
 	update_state_free (subscription->updateState);
