@@ -1,7 +1,7 @@
 /**
  * @file popup_menu.h popup menus
  *
- * Copyright (C) 2003-2011 Lars Windolf <lars.windolf@gmx.de>
+ * Copyright (C) 2003-2024 Lars Windolf <lars.windolf@gmx.de>
  * Copyright (C) 2004-2005 Nathan J. Conrad <t98502@users.sourceforge.net>
  * Copyright (C) 2009 Adrian Bunk <bunk@users.sourceforge.net>
  *
@@ -26,7 +26,6 @@
 #include <gtk/gtk.h>
 
 #include "item.h"
-#include "enclosure.h"
 
 /**
  * Shows a popup menu with options for the item list and the
@@ -34,28 +33,16 @@
  * (Open Link, Copy Item, Copy Link...)
  *
  * @param item	the selected item
- * @param button	the mouse button which was pressed to initiate the event
- * @param activate_time	the time at which the activation event occurred
+ * @param event The event that triggered the popup
  */
-void ui_popup_item_menu (itemPtr item, guint button, guint32 activate_time);
-
-/**
- * Shows a popup menu for the enclosure list view.
- * (Save As, Open With...)
- *
- * @param enclosure	the enclosure
- * @param button	the mouse button which was pressed to initiate the event
- * @param activate_time	the time at which the activation event occurred
- */
-void ui_popup_enclosure_menu (enclosurePtr enclosure, guint button,
-			      guint32 activate_time);
+void ui_popup_item_menu (itemPtr item, const GdkEvent *event);
 
 /* GUI callbacks */
 
 gboolean
 on_mainfeedlist_button_press_event     (GtkWidget       *widget,
-                                        GdkEventButton  *event,
-                                        gpointer         user_data);
+                                        GdkEvent 	*event,
+                                        gpointer 	user_data);
 
 gboolean
 on_mainfeedlist_popup_menu (GtkWidget *widget,
